@@ -2,30 +2,29 @@ source("test_helper.r")
 
 test_should_not_return_named_list <- function() {
   x <- yaml.load("hey: man\n123: 456\n", FALSE)
-  assert_equal(2, length(x))
-  assert_equal(2, length(attr(x, "keys")))
+  assert_equal(2L, length(x))
+  assert_equal(2L, length(attr(x, "keys")))
 
   x <- yaml.load("- dude\n- sup\n- 1.2345", FALSE)
-  assert_equal(3, length(x))
-  assert_equal(0, length(attr(x, "keys")))
+  assert_equal(3L, length(x))
+  assert_equal(0L, length(attr(x, "keys")))
   assert_equal("sup", x[[2]])
 
   x <- yaml.load("dude:\n  - 123\n  - sup", FALSE)
-  assert_equal(1, length(x))
-  assert_equal(1, length(attr(x, "keys")))
-  assert_equal(2, length(x[[1]]))
+  assert_equal(1L, length(x))
+  assert_equal(1L, length(attr(x, "keys")))
+  assert_equal(2L, length(x[[1]]))
 }
 
 test_should_handle_conflicts <- function() {
-  x <- yaml.load("hey: buddy\nhey: guy")
-  assert_equal(1, length(x))
-  assert_equal("buddy", x[[1]])
+  x <- try(yaml.load("hey: buddy\nhey: guy"))
+  assert(inherits(x, "try-error"))
 }
 
 test_should_return_named_list <- function() {
   x <- yaml.load("hey: man\n123: 456\n", TRUE)
-  assert_equal(2, length(x))
-  assert_equal(2, length(names(x)))
+  assert_equal(2L, length(x))
+  assert_equal(2L, length(names(x)))
   assert_equal(c("123", "hey"), sort(names(x)))
   assert_equal("man", x$hey)
 }
@@ -42,29 +41,71 @@ test_should_type_uniform_sequences <- function() {
 
   x <- yaml.load("- hey\n- hi\n- hello")
   assert_equal(c("hey", "hi", "hello"), x)
-
-  x <- yaml.load("- [1, 2]\n- 3\n- [4, 5]")
-  assert_equal(1:5, x)
 }
 
-test_should_merge_maps <- function() {
-  x <- yaml.load("foo: bar\n<<: {baz: boo}", TRUE)
-  assert_equal(2, length(x))
-  assert_equal("boo", x$baz)
-  assert_equal("foo", x$bar)
+test_shows_error_with_tag_type_conflicts <- function() {
+  x <- try(yaml.load("!!str [1, 2, 3]"))
+  assert(inherits(x, "try-error"))
 
-  x <- yaml.load("foo: bar\n<<: [{poo: poo}, {foo: doo}, {baz: boo}]", TRUE)
-  assert_equal(3, length(x))
-  assert_equal("boo", x$baz)
+  x <- try(yaml.load("!!str {foo: bar}"))
+  assert(inherits(x, "try-error"))
+}
+
+test_should_not_collapse_sequences <- function() {
+  x <- yaml.load("- [1, 2]\n- 3\n- [4, 5]")
+  assert_equal(list(1:2, 3L, 4:5), x)
+}
+
+test_should_merge_named_maps <- function() {
+  x <- yaml.load("foo: bar\n<<: {baz: boo}", TRUE)
+  assert_equal(2L, length(x))
   assert_equal("bar", x$foo)
-  assert_equal("poo", x$poo)
+  assert_equal("boo", x$baz)
+
+  x <- yaml.load("foo: bar\n<<: [{quux: quux}, {foo: doo}, {foo: junk}, {baz: blah}, {baz: boo}]", TRUE)
+  assert_equal(3L, length(x))
+  assert_equal("blah", x$baz)
+  assert_equal("bar", x$foo)
+  assert_equal("quux", x$quux)
+
+  x <- yaml.load("foo: bar\n<<: {foo: baz}\n<<: {foo: quux}")
+  assert_equal(1L, length(x))
+  assert_equal("bar", x$foo)
+
+  x <- yaml.load("<<: {foo: baz}\n<<: {foo: quux}\nfoo: bar")
+  assert_equal(1L, length(x))
+  assert_equal("baz", x$foo)
+}
+
+test_should_merge_unnamed_maps <- function() {
+  x <- yaml.load("foo: bar\n<<: {baz: boo}", FALSE)
+  assert_equal(2L, length(x))
+  assert_equal(list("foo", "baz"), attr(x, 'keys'))
+  assert_equal("bar", x[[1]])
+  assert_equal("boo", x[[2]])
+
+  x <- yaml.load("foo: bar\n<<: [{quux: quux}, {foo: doo}, {baz: boo}]", FALSE)
+  assert_equal(3L, length(x))
+  assert_equal(list("foo", "quux", "baz"), attr(x, 'keys'))
+  assert_equal("bar", x[[1]])
+  assert_equal("quux", x[[2]])
+  assert_equal("boo", x[[3]])
+}
+
+test_should_fail_on_duplicate_keys_with_merge <- function() {
+  x <- try(yaml.load("foo: bar\nfoo: baz\n<<: {foo: quux}", TRUE))
+  assert(inherits(x, "try-error"))
 }
 
 test_should_handle_weird_merges <- function() {
-  x <- yaml.load("foo: bar\n<<: [{leet: hax}, blargh, 123]", T)
-  assert_equal(3, length(x))
-  assert_equal("hax", x$leet)
-  assert_equal("blargh", x$`_yaml.merge_`)
+  x <- try(yaml.load("foo: bar\n<<: [{leet: hax}, blargh, 123]", TRUE))
+  assert(inherits(x, "try-error"))
+
+  x <- try(yaml.load("foo: bar\n<<: [123, blargh, {leet: hax}]", TRUE))
+  assert(inherits(x, "try-error"))
+
+  x <- try(yaml.load("foo: bar\n<<: junk", TRUE))
+  assert(inherits(x, "try-error"))
 }
 
 test_should_handle_syntax_errors <- function() {
@@ -77,10 +118,10 @@ test_should_handle_null_type <- function() {
   assert_equal(NULL, x)
 }
 
-test_should_handle_binary_type <- function() {
-  x <- yaml.load("!binary 0b101011")
-  assert_equal("0b101011", x)
-}
+#test_should_handle_binary_type <- function() {
+#  x <- yaml.load("!!binary 0b101011")
+#  assert_equal("0b101011", x)
+#}
 
 test_should_handle_bool_yes_type <- function() {
   x <- yaml.load("yes")
@@ -94,29 +135,34 @@ test_should_handle_bool_no_type <- function() {
 
 test_should_handle_int_hex_type <- function() {
   x <- yaml.load("0xF")
-  assert_equal(15, x)
+  assert_equal(15L, x)
 }
 
 test_should_handle_int_oct_type <- function() {
   x <- yaml.load("015")
-  assert_equal(13, x)
+  assert_equal(13L, x)
 }
 
-test_should_handle_int_base60_type <- function() {
-  x <- yaml.load("1:20")
-  assert_equal("1:20", x)
-}
+#test_should_handle_int_base60_type <- function() {
+#  x <- yaml.load("1:20")
+#  assert_equal("1:20", x)
+#}
 
 test_should_handle_int_type <- function() {
   x <- yaml.load("31337")
-  assert_equal(31337, x)
+  assert_equal(31337L, x)
 }
 
-test_should_handle_float_base60_type <- function() {
-  x <- yaml.load("1:20.5")
-  assert_equal("1:20.5", x)
+test_should_handle_explicit_int_type <- function() {
+  x <- yaml.load("!!int 31337")
+  assert_equal(31337L, x)
 }
 
+#test_should_handle_float_base60_type <- function() {
+#  x <- yaml.load("1:20.5")
+#  assert_equal("1:20.5", x)
+#}
+
 test_should_handle_float_nan_type <- function() {
   x <- yaml.load(".NaN")
   assert_nan(x)
@@ -137,27 +183,27 @@ test_should_handle_float_type <- function() {
   assert_equal(123.456, x)
 }
 
-test_should_handle_timestamp_iso8601_type <- function() {
-  x <- yaml.load("!timestamp#iso8601 2001-12-14t21:59:43.10-05:00")
-  assert_equal("2001-12-14t21:59:43.10-05:00", x)
-}
+#test_should_handle_timestamp_iso8601_type <- function() {
+#  x <- yaml.load("!timestamp#iso8601 2001-12-14t21:59:43.10-05:00")
+#  assert_equal("2001-12-14t21:59:43.10-05:00", x)
+#}
 
-test_should_handle_timestamp_spaced_type <- function() {
-  x <- yaml.load("!timestamp#spaced 2001-12-14 21:59:43.10 -5")
-  assert_equal("2001-12-14 21:59:43.10 -5", x)
-}
+#test_should_handle_timestamp_spaced_type <- function() {
+#  x <- yaml.load("!timestamp#spaced 2001-12-14 21:59:43.10 -5")
+#  assert_equal("2001-12-14 21:59:43.10 -5", x)
+#}
 
-test_should_handle_timestamp_ymd_type <- function() {
-  x <- yaml.load("!timestamp#ymd 2008-03-03")
-  assert_equal("2008-03-03", x)
-}
+#test_should_handle_timestamp_ymd_type <- function() {
+#  x <- yaml.load("!timestamp#ymd 2008-03-03")
+#  assert_equal("2008-03-03", x)
+#}
 
-test_should_handle_timestamp_type <- function() {
-  x <- yaml.load("!timestamp 2001-12-14t21:59:43.10-05:00")
-  assert_equal("2001-12-14t21:59:43.10-05:00", x)
-}
+#test_should_handle_timestamp_type <- function() {
+#  x <- yaml.load("!timestamp 2001-12-14t21:59:43.10-05:00")
+#  assert_equal("2001-12-14t21:59:43.10-05:00", x)
+#}
 
-test_should_handle_merge_type <- function() {
+test_should_handle_alias <- function() {
   x <- yaml.load("- &foo bar\n- *foo")
   assert_equal(c("bar", "bar"), x)
 }
@@ -169,7 +215,9 @@ test_should_handle_str_type <- function() {
 
 test_should_handle_a_bad_anchor <- function() {
   x <- yaml.load("*blargh")
-  print(x)
+  expected <- "_yaml.bad-anchor_"
+  class(expected) <- "_yaml.bad-anchor_"
+  assert_equal(expected, x)
 }
 
 test_should_use_custom_null_handler <- function() {
@@ -242,10 +290,10 @@ test_should_use_custom_timestamp_iso8601_handler <- function() {
   assert_equal("argh!", x)
 }
 
-test_should_use_custom_timestamp_spaced_handler <- function() {
-  x <- yaml.load("!timestamp#spaced 2001-12-14 21:59:43.10 -5", handlers=list("timestamp#spaced"=function(x) { "argh!" }))
-  assert_equal("argh!", x)
-}
+#test_should_use_custom_timestamp_spaced_handler <- function() {
+#  x <- yaml.load('!"timestamp#spaced" 2001-12-14 21:59:43.10 -5', handlers=list("timestamp#spaced"=function(x) { "argh!" }))
+#  assert_equal("argh!", x)
+#}
 
 test_should_use_custom_timestamp_ymd_handler <- function() {
   x <- yaml.load("2008-03-03", handlers=list("timestamp#ymd"=function(x) { "argh!" }))
@@ -262,18 +310,13 @@ test_should_use_custom_str_handler <- function() {
   assert_equal("argh!", x)
 }
 
-test_should_NOT_use_custom_bad_anchor_handler <- function() {
-  x <- yaml.load("*blargh", handlers=list("anchor#bad"=function(x) { "argh!" }))
-  assert_equal("_yaml.bad-anchor_", x)
-}
-
 test_should_use_handler_for_weird_type <- function() {
   x <- yaml.load("!viking pillage", handlers=list(viking=function(x) { paste(x, "the village") }))
   assert_equal("pillage the village", x)
 }
 
 test_should_use_custom_seq_handler <- function() {
-  x <- yaml.load("- 1\n- 2\n- 3", handlers=list(seq=function(x) { as.integer(x) + 3 }))
+  x <- yaml.load("- 1\n- 2\n- 3", handlers=list(seq=function(x) { as.integer(x) + 3L }))
   assert_equal(4:6, x)
 }
 
@@ -283,7 +326,7 @@ test_should_use_custom_map_handler <- function() {
 }
 
 test_should_use_custom_typed_seq_handler <- function() {
-  x <- yaml.load("!foo\n- 1\n- 2", handlers=list(foo=function(x) { as.integer(x) + 1 }))
+  x <- yaml.load("!foo\n- 1\n- 2", handlers=list(foo=function(x) { as.integer(x) + 1L }))
   assert_equal(2:3, x)
 }
 
@@ -292,7 +335,7 @@ test_should_use_custom_typed_map_handler <- function() {
   assert_lists_equal(list(uno="uno", dos="dos"), x)
 }
 
-# NOTE: this works, but it looks like R_tryEval doesn't behave correctly when called in the context of try()
+# NOTE: this works, but R_tryEval doesn't return when called non-interactively
 #test_should_handle_a_bad_handler <- function() {
 #  x <- yaml.load("foo", handlers=list(str=function(x) { blargh }))
 #  str(x)
@@ -307,12 +350,30 @@ test_should_load_empty_documents <- function() {
 
 test_should_load_omap <- function() {
   x <- yaml.load("--- !omap\n- foo:\n  - 1\n  - 2\n- bar:\n  - 3\n  - 4")
-  assert_equal(2, length(x))
+  assert_equal(2L, length(x))
   assert_equal(c("foo", "bar"), names(x))
   assert_equal(1:2, x$foo)
   assert_equal(3:4, x$bar)
 }
 
+test_should_load_omap_without_named <- function() {
+  x <- yaml.load("--- !omap\n- 123:\n  - 1\n  - 2\n- bar:\n  - 3\n  - 4", FALSE)
+  assert_equal(2L, length(x))
+  assert_equal(list(123L, "bar"), attr(x, "keys"))
+  assert_equal(1:2, x[[1]])
+  assert_equal(3:4, x[[2]])
+}
+
+test_should_error_when_named_omap_has_duplicate_key  <- function() {
+  x <- try(yaml.load("--- !omap\n- foo:\n  - 1\n  - 2\n- foo:\n  - 3\n  - 4"))
+  assert(inherits(x, "try-error"))
+}
+
+test_should_error_when_unnamed_omap_has_duplicate_key  <- function() {
+  x <- try(yaml.load("--- !omap\n- foo:\n  - 1\n  - 2\n- foo:\n  - 3\n  - 4", FALSE))
+  assert(inherits(x, "try-error"))
+}
+
 test_should_error_when_omap_is_invalid <- function() {
   x <- try(yaml.load("--- !omap\nhey!"))
   assert(inherits(x, "try-error"))
@@ -332,4 +393,10 @@ test_should_error_for_bad_expressions <- function() {
   assert(inherits(x, "try-error"))
 }
 
+# NOTE: this works, but R_tryEval doesn't return when called non-interactively
+#test_should_error_for_expressions_with_eval_errors <- function() {
+#  x <- try(yaml.load("!expr |\n  1 + non.existent.variable"))
+#  assert(inherits(x, "try-error"))
+#}
+
 source("test_runner.r")
